//
// Copyright (C) 2013-2016  Taiyou Sakashitai, Hiroko Yamada
//
// Calculate Monthlymean 2015/08/10
// Modified for using Julidan day 2016/07/01
//


import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Calendar;

public class monthlymean
{

// Length of the Data in Column/Line Direction
static int X;
static int Y;

// Year/Month/Day
static int YYYY;
static int MM;
static int DD;

// Number of Used Files
static int nFile = 0;

// Number of Used Files for Standard Deviation
static int nFile_std = 0;

// Error Day Threshold
static int Error_DD_THS = 7;

// Present Error Day
static int P_Error_DD;

// Monthlymean
static double MOM[][];

// Standard Deviation
static double STD[][];

// File Type
static String ftype;

// Directory Name to Read Dailymean Data
static String DayMn_DIR;

// Directory Name to Write Dailymean,Monthlymean & STD Data
static String MonthlyMn_DIR;

// TIME Code 
static String TIME_Code;

public static void main(String args[])
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  int I;

  // Filename to Write Monthlymean Data
  String wfilename;

  // Filename to Write Standard Deviation Data
  String std_filename;

  // Year/Month
  YYYY = Integer.valueOf(args[0]).intValue();
  MM   = Integer.valueOf(args[1]).intValue();

  //output/YYYYMM Dir confirm and create
  File newfile = new File("output/" + YYYY + df.format(MM));
   	if (newfile.exists()) {
		System.out.println( newfile + " already exists");
	}else if (newfile.mkdir()){
    		System.out.println("Succeeded to create a directory");
  	}else{
    		System.out.println("Failed to create a directory");
  	}

  date_errorcheck();
  initialization();


  // Calculate Monthlymean
  cal_monthlymean(DayMn_DIR);
  
  String ftype_cut = ftype.substring(0, ftype.length()-4);
 
  wfilename = MonthlyMn_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) +"." + TIME_Code + ftype_cut + ".mm.bin";


  bin bb = new bin();

  // Write Monthlymean Data
  if(nFile > 0)
  {
     bb.d2bin(wfilename, X, Y, MOM);

     // Calculate Standard Deviation
     cal_std(DayMn_DIR);

     std_filename = MonthlyMn_DIR + YYYY + df.format(MM) + "/" +  YYYY + df.format(MM) +"." + TIME_Code + ftype_cut + ".std.bin";

     // Write Standard Deviation Data
     if(nFile_std > 0) bb.d2bin(std_filename, X, Y, STD);

  }//fi


  return;


}//main


public static void cal_monthlymean(String DayMn_DIR)
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );

  // Last Day of the Month
  int Last_DD;

  // Dailymean
  double DM[][] = new double[Y+1][X+1];

  // Filename to Read Dailymean Data
  String filename;

  File file;

  P_Error_DD = 0;

  Last_DD = get_lastday(YYYY, MM);

  bin bb = new bin();
  
  String ftype_cut = ftype.substring(0, ftype.length()-4);

  // Calculate Sum of Dailymean Data
  for(DD=1;DD<=Last_DD;DD++)
  {
     filename = DayMn_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) + df.format(DD) +"." + TIME_Code  + ftype_cut + ".dm.bin";

     file = new File(filename);

     if(file.exists())
     {
        DM = bb.bin2d(filename, X, Y);
        add_data(DM);
        nFile++;

     }else{

        P_Error_DD++;
        System.out.println(filename + " is not found");

     }//fi

  }//DD


  if(P_Error_DD >= Error_DD_THS)
  {
     System.out.println("Loss of dailymean is too much on " + YYYY + df.format(MM));
     System.exit(1);

  }//fi


  // Calculate Monthlymean
  if(nFile > 0) MOM = cal_mean_d(MOM, nFile);


  return;


}//cal_monthlymean


public static void cal_std(String DayMn_DIR)
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );

  // Last Day of the Month
  int Last_DD;

  // Data Read
  float DR[][] = new float[Y+1][X+1];

  // Filename of Original Data
  String filename;

  File file;


  Last_DD = get_lastday(YYYY, MM);


  bin bb = new bin();

  String ftype_cut = ftype.substring(0, ftype.length()-4);

  // Calculate Variance
  for(DD=1;DD<=Last_DD;DD++)
  {
     filename = DayMn_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) + df.format(DD) +"." + TIME_Code + ftype_cut + ".dm.bin";

     file = new File(filename);

     if(file.exists())
     {
        DR = bb.bin2f(filename, X, Y);
        cal_var(DR);
        nFile_std++;

     }//fi
  }//DD

  // Calculate Standard Deviation
  if(nFile_std > 0) 
  {
     STD = cal_mean_d(STD, nFile_std);
     standard_deviation();

  }//fi


  return;


}//cal_std


public static void add_data(double DR[][])
{

  int I,J;

  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        MOM[I][J] = MOM[I][J] + DR[I][J];

     }//J
  }//I


  return;


}//add_data


public static double[][] cal_mean_d(double DD[][], int N)
{

  int I,J;

  // Result
  double RR[][] = new double[Y+1][X+1];


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[I][J] = DD[I][J] / N;

     }//J
  }//I


  return RR;


}//cal_mean_d


public static void cal_var(float DR[][])
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[I][J] = STD[I][J] + (DR[I][J] - MOM[I][J]) * (DR[I][J] - MOM[I][J]);

     }//J
  }//I


  return;


}//cal_var


public static void standard_deviation()
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[I][J] = Math.sqrt(STD[I][J]);

     }//J
  }//I


  return;


}//standard_deviation


public static int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;

  Calendar time = Calendar.getInstance();

  time.set(Calendar.YEAR, Year);
  time.set(Calendar.MONTH, Month-1);
  time.set(Calendar.DAY_OF_MONTH, 1);
  time.set(Calendar.HOUR_OF_DAY, 0);
  time.set(Calendar.MINUTE, 0);

  Last_DD = time.getActualMaximum(Calendar.DAY_OF_MONTH);


  return Last_DD;


}//get_lastday


public static void date_errorcheck()
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     System.out.println("Year " + YYYY + " is less than or equal to 0");
     System.exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     System.out.println("Month " + MM + " does not exist");
     System.exit(1);

  }//fi
  

  return;


}//date_errorcheck


public static void initialization()
{

  read_formatfile();


  MOM = new double[Y+1][X+1];
  STD = new double[Y+1][X+1];


  return;


}//initialization


static public void read_formatfile()
{

  // Present Line
  int pline=1;

  String filename;
  String readline;
  String param[] = new String [100+1]; 


  filename = "config/format.txt";


  // Read Format File
  try
  {
       
    System.out.print(filename +"  Loading ...");

    BufferedReader br = new BufferedReader(new FileReader(filename));

    while( (readline = br.readLine()) != null ){

        StringTokenizer st = new StringTokenizer(readline);

	 param[pline] =  st.nextToken(", ");

        pline++;
                 
    }//while

    br.close();
	
  }catch (IOException e){

      System.err.println("Initialize error\n");
      System.exit(1);

   }//try

  System.out.println(" finish");

  
  TIME_Code = param[2];

  X = Integer.valueOf(param[3]).intValue();
  Y = Integer.valueOf(param[4]).intValue();

  ftype = param[5];
  DayMn_DIR = param[7];

  MonthlyMn_DIR = param[11];


  return;


}//read_formatfile


}//EOF

